// 
// Modified from NinjaTrader(v6.5.1000.8) indicator "TickCounter".
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// The Tick Counter displays the current tick count of a bar.
	/// Modified from NinjaTrader(v6.5.1000.8) indicator "TickCounter" to add more display options.
	/// </summary>
	[Description("Displays the tick count of a bar, with more display options.")]
	public class TickCounter2 : Indicator
	{
		#region Variables
		private bool			countDown		= true;
		private bool			showPercent		= false;
		private StringFormat	stringFormat	= new StringFormat();
		private SolidBrush		textBrush		= new SolidBrush(Color.Black);
		private Font			textFont		= new Font("Arial", 10);
		private float			textWidth		= 0;
		private float			textHeight		= 0;
		private float 		noTickTextWidth 	= 0;
		private float 		noTickTextHeight 	= 0;
		private	bool			doOnce			= false;			
		private	Color			textColor		= Color.Black;
		private	bool			showNumbersOnly	= false;
		private	DisplayLocation	dispLocation	= DisplayLocation.Bottom_Right;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			ChartOnly			= true;
			Overlay				= true;
			CalculateOnBarClose = false;
		}

		/// <summary>
		/// </summary>
		protected override void OnBarUpdate(){}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			if (Bars == null)
				return;

			// Assign textBrush color and calculate the string size (only do this once)
			if (!doOnce)
			{
				
				doOnce = true;
				textBrush.Color = TextColor;
				SizeF size = graphics.MeasureString(((ShowNumbersOnly || DispLocation.ToString() == "Float_with_price") ? "%" : CountDown ? "Ticks remaining = %" : "Tick count = %") + Bars.Period.Value, TextFont);
				textWidth		= size.Width + 5;
				textHeight		= size.Height + 5;

				SizeF noTickSize = graphics.MeasureString("Tick Counter only works on tick based interval", TextFont);
				noTickTextWidth = noTickSize.Width + 5;
				noTickTextHeight = noTickSize.Height + 5;
			}

			// Only do this on Tick charts
			if (Bars.Period.Id == PeriodType.Tick)
			{
				// calculate x,y location for text display
				float x;
				float y;
				switch (DispLocation)
				{
					case DisplayLocation.Float_with_price:
						x = bounds.X + bounds.Width - ChartControl.BarMarginRight + 5;
						y = (int)(((bounds.Y + bounds.Height) - ((Close[0] - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height) - textHeight*0.4);
						if(ChartControl.BarMarginRight < textWidth-12)
						graphics.DrawString("Increase chart Properties - Right side margin", TextFont, textBrush, bounds.X + bounds.Width - noTickTextWidth, bounds.Y + bounds.Height - noTickTextHeight, stringFormat);
						break;
					case DisplayLocation.Bottom_Right:
						x = bounds.X + bounds.Width - textWidth;
						y = bounds.Y + bounds.Height - textHeight;
						break;
					case DisplayLocation.Top_Right:
						x = bounds.X + bounds.Width - textWidth;
						y = bounds.Y + 12;
						break;
					case DisplayLocation.Bottom_Left:
						x = bounds.X;
						y = bounds.Y + bounds.Height - textHeight - 5;
						break;
					case DisplayLocation.Top_Left:
						x = bounds.X;
						y = bounds.Y + 12;
						break;
					default: // Bottom_Right
						x = bounds.X + bounds.Width - textWidth;
						y = bounds.Y + bounds.Height - textHeight;
						break;
				}
				// calculate the tick count value
				double tickCount = ShowPercent ? CountDown ? (int)((1 - Bars.PercentComplete) * 100) : (int)(Bars.PercentComplete * 100) : CountDown ? Bars.Period.Value - Bars.TickCount : Bars.TickCount;
				// draw the text
				if(ShowNumbersOnly || DispLocation.ToString() == "Float_with_price")
					graphics.DrawString(tickCount + (ShowPercent ? "%" : ""), TextFont, textBrush, x, y, stringFormat);
				else
					graphics.DrawString((CountDown ? " Ticks remaining = " + tickCount : "Tick count = " + tickCount) + (ShowPercent ? "%" : ""), textFont, textBrush, x, y, stringFormat);
			}
			else
				graphics.DrawString("Tick Counter only works on tick based intervals", TextFont, textBrush, bounds.X + bounds.Width - noTickTextWidth, bounds.Y + bounds.Height - noTickTextHeight, stringFormat);
		}

		// Trim the indicator's label as the font parameters are very long
		public override string ToString()
		{
			return Name + "(" + CountDown + "," + DispLocation + "," + ShowNumbersOnly + "," + ShowPercent + ")";
		}
		
		#region Properties
		/// <summary>
		/// </summary>
		[Gui.Design.VisualizationOnly()]
		[Description("Indicates if the indicator displays remaining ticks or current ticks of a bar.")]
		[Category("Parameters")]
		[Gui.Design.DisplayName("Count down")]
		public bool CountDown
		{
			get { return countDown; }
			set { countDown = value; }
		}

		[Gui.Design.VisualizationOnly()]
		[Description("Shows a percent value instead of absolute value.")]
		[Category("Parameters")]
		[Gui.Design.DisplayName("Show percent")]
		public bool ShowPercent
		{
			get { return showPercent; }
			set { showPercent = value; }
		}
		
		[XmlIgnore()]
		[Description("Select color for text")]
		[Category("Parameters")]
		[Gui.Design.DisplayName("Text color")]
		public Color TextColor
		{
			get { return textColor; }
			set { textColor = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string TextColorSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(textColor); }
			set { textColor = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		[XmlIgnore()]
		[Description("Select font for text")]
		[Category("Parameters")]
		[Gui.Design.DisplayName("Text font")]
		public Font TextFont
		{
			get { return textFont; }
			set { textFont = value; }
		}

		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string TextFontSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableFont.ToString(textFont); }
			set { textFont = NinjaTrader.Gui.Design.SerializableFont.FromString(value); }
		}
		
		[Gui.Design.VisualizationOnly()]
		[Description("Display only the numerical value, and omit the descriptive text")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("Show numbers only")]
        public bool ShowNumbersOnly
        {
            get { return showNumbersOnly; }
            set { showNumbersOnly = value; }
        }
		
		[Description("Select location for text display (select from drop-down list)")]
        [Category("Parameters")]
		[Gui.Design.DisplayName("Display Position")]
        public DisplayLocation DispLocation
        {
            get { return dispLocation; }
            set { dispLocation = value; }
        }
        #endregion
	}
}

public enum DisplayLocation
{
	Bottom_Right,
	Top_Right,
	Bottom_Left,
	Top_Left,
	Float_with_price,
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TickCounter2[] cacheTickCounter2 = null;

        private static TickCounter2 checkTickCounter2 = new TickCounter2();

        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        public TickCounter2 TickCounter2(bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            return TickCounter2(Input, countDown, dispLocation, showNumbersOnly, showPercent, textColor, textFont);
        }

        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        public TickCounter2 TickCounter2(Data.IDataSeries input, bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            checkTickCounter2.CountDown = countDown;
            countDown = checkTickCounter2.CountDown;
            checkTickCounter2.DispLocation = dispLocation;
            dispLocation = checkTickCounter2.DispLocation;
            checkTickCounter2.ShowNumbersOnly = showNumbersOnly;
            showNumbersOnly = checkTickCounter2.ShowNumbersOnly;
            checkTickCounter2.ShowPercent = showPercent;
            showPercent = checkTickCounter2.ShowPercent;
            checkTickCounter2.TextColor = textColor;
            textColor = checkTickCounter2.TextColor;
            checkTickCounter2.TextFont = textFont;
            textFont = checkTickCounter2.TextFont;

            if (cacheTickCounter2 != null)
                for (int idx = 0; idx < cacheTickCounter2.Length; idx++)
                    if (cacheTickCounter2[idx].CountDown == countDown && cacheTickCounter2[idx].DispLocation == dispLocation && cacheTickCounter2[idx].ShowNumbersOnly == showNumbersOnly && cacheTickCounter2[idx].ShowPercent == showPercent && cacheTickCounter2[idx].TextColor == textColor && cacheTickCounter2[idx].TextFont == textFont && cacheTickCounter2[idx].EqualsInput(input))
                        return cacheTickCounter2[idx];

            TickCounter2 indicator = new TickCounter2();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.CountDown = countDown;
            indicator.DispLocation = dispLocation;
            indicator.ShowNumbersOnly = showNumbersOnly;
            indicator.ShowPercent = showPercent;
            indicator.TextColor = textColor;
            indicator.TextFont = textFont;
            indicator.SetUp();

            TickCounter2[] tmp = new TickCounter2[cacheTickCounter2 == null ? 1 : cacheTickCounter2.Length + 1];
            if (cacheTickCounter2 != null)
                cacheTickCounter2.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTickCounter2 = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TickCounter2 TickCounter2(bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            return _indicator.TickCounter2(Input, countDown, dispLocation, showNumbersOnly, showPercent, textColor, textFont);
        }

        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        public Indicator.TickCounter2 TickCounter2(Data.IDataSeries input, bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            return _indicator.TickCounter2(input, countDown, dispLocation, showNumbersOnly, showPercent, textColor, textFont);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TickCounter2 TickCounter2(bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            return _indicator.TickCounter2(Input, countDown, dispLocation, showNumbersOnly, showPercent, textColor, textFont);
        }

        /// <summary>
        /// Displays the tick count of a bar, with more display options.
        /// </summary>
        /// <returns></returns>
        public Indicator.TickCounter2 TickCounter2(Data.IDataSeries input, bool countDown, DisplayLocation dispLocation, bool showNumbersOnly, bool showPercent, Color textColor, Font textFont)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TickCounter2(input, countDown, dispLocation, showNumbersOnly, showPercent, textColor, textFont);
        }

    }
}
#endregion
